from django.db import models


class BasicModel(models.Model):
    class Meta:
        app_label = 'local_app'

    first_name = models.CharField(max_length=20)
    start = models.DateField()


class ChildModel(BasicModel):
    class Meta:
        app_label = 'local_app'

    last_name = models.CharField(max_length=20)
    nick_name = models.CharField(max_length=20)


class GrandchildModel(ChildModel):
    class Meta:
        app_label = 'local_app'

    _middle_name_field = models.CharField(max_length=20, db_column='middle_name')
    end = models.DateField()



"""
Management command to populate initial history.
"""
import logging
import time
from datetime import datetime

from django.apps import apps
from django.core.management.base import BaseCommand
from django.db import connection, transaction

get_model = apps.get_model

log = logging.getLogger(__name__)


class Command(BaseCommand):
    """
    Populate initial history for models using django-simple-history.
    Example usage:
    $ ./manage.py lms generate_history --tables organizations_organization entitlements_courseentitlement \
      --batchsize 1000 --sleep_between 1 --settings=devstack
    """

    help = (
        "Populates the corresponding historical records with"
        "the current state of records using django-simple-history"
        "which do not have a historical record yet."
    )

    DEFAULT_BATCH_SIZE = 200
    DEFAULT_SLEEP_BETWEEN_INSERTS = 1
    HISTORY_DATE = datetime.today().strftime('%Y-%m-%d')
    HISTORY_USER_ID = 'NULL'
    HISTORY_CHANGE_REASON = 'initial history population'

    def add_arguments(self, parser):
        super().add_arguments(parser)

        parser.add_argument("--tables", nargs="*", type=str)

        parser.add_argument(
            '--sleep_between',
            default=self.DEFAULT_SLEEP_BETWEEN_INSERTS,
            type=float,
            help='Seconds to sleep between chunked inserts.'
        )

        parser.add_argument(
            "--batchsize",
            action="store",
            default=self.DEFAULT_BATCH_SIZE,
            type=int,
            help="Maximum number of history rows to insert in each batch.",
        )

    def columns_from_schema(self, cursor, table):
        query = """
            SELECT
                column_name
            FROM information_schema.columns
            WHERE table_name='{}'
            ORDER BY ordinal_position
            """.format(table)
        cursor.execute(query)
        columns = [column[0] for column in cursor.fetchall()]
        return columns

    def handle(self, *args, **options):
        tables = options.get("tables", [])
        increment = options['batchsize']
        sleep_between = options['sleep_between']

        for table in tables:
            # This assumes the model is using default historical model naming convention.
            # The management command would fail if the historical table does not exist.
            historical_table = '_historical'.join(table.rsplit('_', 1))
            with connection.cursor() as cursor:
                query = """
                    SELECT
                        MIN(t.id),
                        MAX(t.id)
                    FROM {table} t
                    LEFT JOIN {historical_table}
                        ON t.id = {historical_table}.id
                    WHERE {historical_table}.id IS NULL
                    """.format(
                        table=table,
                        historical_table=historical_table,
                )
                cursor.execute(query)
                start_id, end_id = cursor.fetchone()
                if not start_id or not end_id:
                    log.info("No records with missing historical records for table %s - skipping.", table)
                    continue
                columns = self.columns_from_schema(cursor, table)
            while True:
                with transaction.atomic():
                    with connection.cursor() as cursor:
                        log.info(
                            "Inserting historical records for %s starting with id %s to %s",
                            table,
                            start_id,
                            start_id + increment - 1,
                        )
                        # xss-lint: disable=python-wrap-html
                        query = """
                            INSERT INTO {historical_table}(
                                {insert_columns},history_date,history_change_reason,history_type,history_user_id
                            )
                            SELECT {select_columns},'{history_date}','{history_change_reason}', '+', {history_user_id}
                            FROM {table} t
                            LEFT JOIN {historical_table}
                                ON t.id={historical_table}.id
                            WHERE {historical_table}.id IS NULL
                                AND t.id >= {start_id}
                                AND t.id < {end_id}
                            """.format(
                                table=table,
                                historical_table=historical_table,
                                # this cmd fails for tables containing reserved keywords in column.
                                # https://dev.mysql.com/doc/refman/5.5/en/glossary.html
                                # Backticked columns to avoid MYSQL errors
                                insert_columns=','.join([f'`{c}`' for c in columns]),
                                select_columns=','.join([f't.`{c}`' for c in columns]),
                                history_date=self.HISTORY_DATE,
                                history_change_reason=self.HISTORY_CHANGE_REASON,
                                history_user_id=self.HISTORY_USER_ID,
                                start_id=start_id,
                                end_id=start_id + increment
                        )
                        log.info(query)
                        count = cursor.execute(query)
                        log.info("Inserted %s historical records", count)
                start_id += increment
                log.info("Sleeping %s seconds...", sleep_between)
                time.sleep(sleep_between)
                if start_id > end_id:
                    break



from django.db import models


class BasicModel(models.Model):
    class Meta:
        app_label = 'non_concrete_app'

    first_name = models.CharField(max_length=20)


class ModelMixin:
    age = 100


class MixedModel(BasicModel, ModelMixin):
    class Meta:
        app_label = 'non_concrete_app'

    last_name = models.CharField(max_length=20)


class AbstractModel(models.Model):
    class Meta:
        app_label = 'non_concrete_app'
        abstract = True

    start_date = models.CharField(max_length=20)


class ModelWithAbstractParent(AbstractModel):
    class Meta:
        app_label = 'non_concrete_app'

    end_date = models.CharField(max_length=20)


class ProxyModel(BasicModel):
    class Meta:
        app_label = 'non_concrete_app'
        proxy = True



"""
check_reserved_keywords.py

A utility for analyzing a Django application to see if any of the model fields
have names that are on a list of reseverd keywords. Certain tools used for downstream
data storage and analysis, like Stitch, have restrictions for which column names they
allow. Modifying existing field names in a live Django application is a complicated,
long and potentially risky operation. This tool allows you to prevent new fields from
being added to the application that would break these rules. It does so by loading the
app and inspecting the models of both the locally defined models and third party
installed apps.

EXAMPLE USAGE:

python manage.py check_reserved_keywords.py \
    --reserved_keyword_file reserved_keywords.yml \
    --override_file overrides.yml \
    --report_path reports \
    --report_file keyword_report.csv \
    --system MYSQL

OPTIONS:

* --reserved_keyword_file: pass a yaml file containing a mapping of tool (i.e. Snowflake) to a
  list of reserved keywords for that technology
* --override_file: pass a yaml file containing a list of model fields to ignore when scanning
  for reserved keyword violations.
* --report_path: pass a path in which to write a csv report of the violations found with this script.
* --report_file: name of file in which to write violation report data
* --system: the type of database management system (MYSQL, SNOWFLAKE), as defined in
  `reserved_keyword_file`, for which you will be scanning for reserved keywords.

"""

import argparse
import inspect
import io
import logging
import os
import sys
from os.path import dirname

import django
import six
import yaml
from django.apps import apps
from django.core.management import CommandError
from django.core.management.base import BaseCommand
from django.db import models

if six.PY2:
    FileNotFoundError = IOError

log = logging.getLogger(__name__)


class Violation:
    """
    A Django model field name that is in conflict with a defined list of reserved keywords
    """

    def __init__(self, model, field_name, system, override=False):
        self.model = model
        self.field = field_name
        self.system = system
        self.override = override

    def __str__(self):
        return "{} conflict in {}:{}:{}.{}".format(
            self.system,
            self.model._meta.app_label,
            self.module_name,
            self.model._meta.concrete_model.__name__,
            self.field
        )

    @property
    def module_name(self):
        """
        The path to the module containing the reserved keyword violation
        """
        module_string = self.model._meta.concrete_model.__module__.replace('.', '/')
        return f"{module_string}.py"

    @property
    def inherited(self):
        """
        Return whether or not this violation is defined in a parent model of the model
        in which it was found
        """
        return self.field not in [f.name for f in self.model._meta.local_fields]

    @property
    def local_app(self):
        """
        Return whether or not this violation was found in a local source file or a package
        installed in the Python environment running this code. In this case, 'local' means
        in the source code files in the directory that this is being run in. This can be
        determined by checking if the Django app containing this violation is in env
        """
        # This will return the path to the virtualenv/python installation being used to
        # run this django app (i.e. /edx/app/credentials/venvs/credentials)
        env_base_path = dirname(dirname(sys.executable))
        app_path = apps.get_app_config(self.model._meta.app_label).path
        return env_base_path not in app_path


class ConfigurationException(Exception):
    pass


class Config:
    """
    A collection of configuration data used throughout this script
    """

    def __init__(self, reserved_keyword_config_file, override_file, report_path, report_file, system):
        reserved_keyword_config = self.read_config_file(reserved_keyword_config_file)
        if system:
            try:
                self.reserved_keyword_config = {system: reserved_keyword_config[system]}
            except KeyError:
                raise ConfigurationException(
                    f"Parameter {system} missing from config file {reserved_keyword_config_file}"
                )
        else:
            self.reserved_keyword_config = reserved_keyword_config
        self.override_file = override_file
        if override_file:
            self.overrides = self.read_config_file(override_file)
        else:
            self.overrides = {}
        self.validate_override_config()
        self.report_path = report_path
        self.report_file = os.path.join(report_path, report_file)

    @staticmethod
    def read_config_file(config_file_path):
        log.info("Loading config file: {}".format(config_file_path))
        try:
            config_dict = yaml.safe_load(config_file_path)
            # for ease of use later in this script, change keys without values from None to empty lists
            for key in list(config_dict.keys()):
                if not config_dict[key]:
                    config_dict[key] = []
        except (yaml.parser.ParserError, FileNotFoundError):
            raise ConfigurationException(f"Unable to load config file: {config_file_path}")
        if not config_dict:
            raise ConfigurationException(f"Config file is empty: {config_file_path}")
        return config_dict

    def validate_override_config(self):
        invalid_chars = [' ', ',', '-']
        def check(s): return any([c in invalid_chars for c in s])
        for system, override_list in list(self.overrides.items()):
            for pattern in override_list:
                try:
                    model_name, field_name = pattern.split('.')
                    if not model_name[0].isupper():
                        log.error('Model names must be camel case')
                        raise ValueError()
                    if check(field_name) or check(model_name):
                        log.error('Invalid character found')
                        raise ValueError()
                except ValueError:
                    raise ConfigurationException(f"Invalid value in override file: {pattern}")


def collect_concrete_models():
    """
    Walk through all of the INSTALLED_APPS in a Django project, gathering all
    of the 'concrete' models. In this case, 'concrete' refers to the fact
    that a model has a corresponding table within a database (as opposed to
    an abstract model, which does not). For more information, see:
    https://openedx.atlassian.net/wiki/spaces/PLAT/pages/895287378/OEP-30+Implementation
    """
    def is_concrete(model):
        return (
            issubclass(model, models.Model) and
            model is not models.Model and
            not model._meta.abstract and
            not model._meta.proxy
        )

    concrete_models = set()

    log.info("Collecting all concrete models in installed apps")
    for app in django.apps.apps.get_app_configs():
        log.info("Inspecting app: {}".format(app.label))
        app_models = []
        for root_model in app.get_models():

            model_hierarchy = inspect.getmro(root_model)
            for model in model_hierarchy:
                if is_concrete(model):
                    model_name = model._meta.object_name
                    concrete_models.add(model)
                    app_models.append(model_name)
        if app_models:
            log.info("Found models: {}".format(','.join(app_models)))

    log.info("Collected {} concrete models".format(len(concrete_models)))
    return list(concrete_models)


def get_fields_per_model(model):
    """
    Given a model, return a list of all of the field names on the model,
    regardless of whether they are explicitly present or present through
    inheritance. Do not include hidden fields, as these are not created
    in app code.
    """
    def _get_db_field_name(field):
        """
        Get the actual name that will be used to name the column for this
        model field. These can be overridden with the 'db_column' field.
        """
        if hasattr(field, 'db_column') and field.db_column is not None:
            return field.db_column
        else:
            return field.column

    return [
        _get_db_field_name(f)
        for f in model._meta.get_fields(include_hidden=False)
        if not f.auto_created
    ]


def check_model_for_violations(model, config):
    """
    See if any of the fields in a given model are in conflict with the list
    of reserved keyword names. Return a list of any such violations.
    """
    violations = []

    for field in get_fields_per_model(model):
        for system in list(config.reserved_keyword_config.keys()):
            reserved_keywords = [rkw.lower() for rkw in config.reserved_keyword_config[system]]
            if field in reserved_keywords:
                full_field_name = "{}.{}".format(
                    model._meta.concrete_model.__name__,
                    field
                )

                if system in list(config.overrides.keys()) and full_field_name in config.overrides[system]:
                    override = True
                else:
                    override = False

                violation = Violation(model, field, system, override)
                violations.append(violation)
                if override:
                    log.warn("Violation detected but on whitelist: {}".format(violation))
                else:
                    log.error("Violation detected: {}".format(violation))
    return violations


def generate_report(violations, config):
    """
    Generate a csv file report for the violations that were detected.
    """
    valid_violation_strings = sorted([str(v) for v in violations if not v.override])
    overridden_violation_strings = sorted([str(v) for v in violations if v.override])

    if not os.path.isdir(config.report_path):
        os.mkdir(config.report_path)
    log.info("Writing report to {}".format(config.report_file))
    with open(config.report_file, 'w') as report_file:
        report_file.write(f"Using override_file: {config.override_file.name}\n\n")

        report_file.write("The following violations were detected:\n")
        report_file.write("---------------------------------------\n")
        for violation in valid_violation_strings:
            report_file.write(f"{violation}\n")

        report_file.write("\n\n")
        report_file.write("The following violations were detected, but were overridden:\n")
        report_file.write("------------------------------------------------------------\n")
        for violation in overridden_violation_strings:
            report_file.write(f"{violation}\n")
    log.info(
        "Successfully wrote {} violations to report".format(len(violations))
    )


def set_status(violations, config):
    """
    set the exit code of this script, depending on whether or not there are
    any reserved keyword Violations detected that are not on the override
    list
    """
    valid_violations = list(
        [v for v in violations if not v.override]
    )
    violation_count = len(valid_violations)
    if violation_count > 0:
        raise CommandError(f"Found {violation_count} reserved keyword conflicts!")
    else:
        log.info("No reserved keyword conflicts detected")


class writable_dir(argparse.Action):
    """
    A custom argparse action to check that a given argument value represents a writable directory.
    """

    def __call__(self, parser, namespace, values, option_string=None):
        prospective_dir = values
        if not os.path.isdir(prospective_dir):
            raise argparse.ArgumentTypeError(f"{prospective_dir} is not a valid path")
        elif not os.access(prospective_dir, os.W_OK):
            raise argparse.ArgumentTypeError(f"{prospective_dir} is not a readable dir")
        else:
            setattr(namespace, self.dest, prospective_dir)


class Command(BaseCommand):
    help = (
        'Check a Django application to see if any of the model fields have names that are on a list of reseverd '
        'keywords.'
    )

    def add_arguments(self, parser):
        parser.add_argument(
            '--reserved_keyword_file',
            default=os.path.join(os.path.dirname(__file__), 'default_reserved_keywords.yml'),
            help='Path to the configuration file containing the lists of reserved keywords to check for.',
            type=argparse.FileType('r', encoding='UTF-8'),
        )
        parser.add_argument(
            '--override_file',
            default=None,
            help=(
                'Path to the configuration file containing the lists of '
                'reserved keywords that can be excluded from analysis'
            ),
            type=argparse.FileType('r', encoding='UTF-8'),
        )
        parser.add_argument(
            '--report_path',
            default='reports',
            help='Path to write a report file containing all of the reserved keyword violations',
            action=writable_dir,
        )
        parser.add_argument(
            '--report_file',
            default='reserved_keyword_report.csv',
            help='Name of file used for writing report data',
        )
        parser.add_argument(
            '--system',
            default=None,
            help="Specific system list in 'reserved_keyword_file' to check for (i.e. MYSQL)"
        )

    def handle(self, *args, **options):
        config = Config(
            options['reserved_keyword_file'], options['override_file'],
            options['report_path'], options['report_file'], options['system']
        )
        concrete_models = collect_concrete_models()
        violations = []
        log.info("Checking models for reserved keyword violations")
        for model in concrete_models:
            violations += check_model_for_violations(model, config)
        generate_report(violations, config)
        set_status(violations, config)



#!/usr/bin/env python
"""
Setup script for the edx-django-release-util package.
"""

import io

from setuptools import find_packages, setup

import release_util


def load_requirements(*requirements_paths):
    """
    Load all requirements from the specified requirements files.

    Returns:
        list: Requirements file relative path strings
    """
    requirements = set()
    for path in requirements_paths:
        requirements.update(
            line.split('#')[0].strip() for line in open(path).readlines()
            if is_requirement(line.strip())
        )
    return list(requirements)


def is_requirement(line):
    """
    Return True if the requirement line is a package requirement.

    Returns:
        bool: True if the line is not blank, a comment, a URL, or an included file
    """
    return line and not line.startswith(('-r', '#', '-e', 'git+', '-c'))


LONG_DESCRIPTION = open('README.rst', encoding='utf-8').read()

setup(
    name='edx-django-release-util',
    version=release_util.__version__,
    description='edx-django-release-util',
    author='edX',
    author_email='oscm@edx.org',
    long_description=LONG_DESCRIPTION,
    license='AGPL 3.0',
    url='http://github.com/edx/edx-django-release-util',
    install_requires=load_requirements('requirements/base.in'),
    packages=find_packages(exclude=['*.test', '*.tests']),
    include_package_data=True,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Environment :: Web Environment',
        'Topic :: Internet',
        'License :: OSI Approved :: GNU Affero General Public License v3',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.8',
        'Framework :: Django',
        'Framework :: Django :: 2.2',
    ],
)



import io
import os
import tempfile

import django
import pytest
import yaml
from django.apps import apps
from django.conf import settings
from django.core.management import CommandError

from release_util.management.commands.check_reserved_keywords import (
    Config,
    ConfigurationException,
    Violation,
    check_model_for_violations,
    collect_concrete_models,
    get_fields_per_model,
    set_status,
)


@pytest.fixture
def django_setup():
    os.environ['DJANGO_SETTINGS_MODULE'] = 'release_util.tests.test_check_reserved_keywords.test_app.test_app.settings'
    django.setup()


def load_apps(app_names):
    settings.INSTALLED_APPS = app_names
    apps.ready = False
    apps.apps_ready = apps.models_ready = apps.loading = apps.ready = False
    apps.app_configs = {}
    apps.populate(settings.INSTALLED_APPS)


def test_concrete_model_collection(django_setup):
    load_apps(['release_util.tests.test_check_reserved_keywords.test_app.local_app'])
    models = collect_concrete_models()
    expected_model_names = ['BasicModel', 'ChildModel', 'GrandchildModel']
    assert sorted([m._meta.concrete_model.__name__ for m in models]) == expected_model_names


def test_concrete_model_collection_with_third_party_apps(django_setup):
    load_apps([
        'django.contrib.contenttypes',
        'django.contrib.auth',
        'waffle'
    ])
    models = collect_concrete_models()
    expected_model_names = [
        'ContentType', 'Flag', 'Group', 'Permission', 'Sample', 'Switch', 'User'
    ]
    assert sorted([m._meta.concrete_model.__name__ for m in models]) == expected_model_names


def test_model_collection_with_non_concrete_models(django_setup):
    load_apps(['release_util.tests.test_check_reserved_keywords.test_app.non_concrete_app'])
    models = collect_concrete_models()
    expected_model_names = ['BasicModel', 'MixedModel', 'ModelWithAbstractParent']
    assert sorted([m._meta.concrete_model.__name__ for m in models]) == expected_model_names


def test_field_collection_with_inheritance(django_setup):
    from .test_app.local_app import models as local_models
    model = local_models.GrandchildModel
    model_fields = get_fields_per_model(model)
    expected_field_names = [
        'end', 'first_name', 'last_name', 'middle_name', 'nick_name', 'start'
    ]
    assert sorted(model_fields) == expected_field_names


def test_field_collection_with_non_concrete_parents(django_setup):
    from .test_app.non_concrete_app import models as non_concrete_models
    model = non_concrete_models.ModelWithAbstractParent
    model_fields = get_fields_per_model(model)
    expected_field_names = [
        'end_date', 'start_date'
    ]
    assert sorted(model_fields) == expected_field_names


def test_field_collection_with_third_party_app(django_setup):
    load_apps([
        'django.contrib.contenttypes',
        'django.contrib.auth',
        'waffle'
    ])
    from waffle.models import Switch
    model_fields = get_fields_per_model(Switch)
    expected_field_names = ['active', 'created', 'modified', 'name', 'note']
    assert sorted(model_fields) == expected_field_names


def test_local_app_location_detection(django_setup):
    from .test_app.local_app import models as local_models
    load_apps(['release_util.tests.test_check_reserved_keywords.test_app.local_app'])
    violation = Violation(local_models.GrandchildModel, None, None, None)
    assert violation.local_app


def test_third_party_app_location_detection(django_setup):
    """
    django-waffle is considered to be not-local, that is, it is not defined within
    the local source code of this application, but is installed in its Python
    environment
    """
    load_apps([
        'django.contrib.contenttypes',
        'django.contrib.auth',
        'waffle'
    ])
    from waffle.models import Switch
    violation = Violation(Switch, None, None, None)
    assert not violation.local_app


def test_invalid_override_config():
    with pytest.raises(ConfigurationException) as exception:
        keyword_file = open(
            'release_util/tests/test_check_reserved_keywords/test_files/reserved_keywords.yml', 'r'
        )
        override_file = open(
            'release_util/tests/test_check_reserved_keywords/test_files/invalid_overrides.yml', 'r'
        )
        Config(keyword_file, override_file, 'reports', 'report.csv', None)
    exc_msg = str(exception.value)
    assert "Invalid value in override file: BasicModel. second_field" in exc_msg


def test_reserved_keyword_detection(django_setup):
    from .test_app.local_app import models as local_models
    load_apps(['release_util.tests.test_check_reserved_keywords.test_app.local_app'])
    model = local_models.GrandchildModel
    keyword_file = open('release_util/tests/test_check_reserved_keywords/test_files/reserved_keywords.yml', 'r')
    config = Config(keyword_file, None, 'reports', 'report.csv', None)
    violations = check_model_for_violations(model, config)
    violation_strings = sorted([str(v) for v in violations])
    expected_violations = [
        (
            'MYSQL conflict in local_app:release_util/tests/test_check_reserved_keywords/test_app/'
            'local_app/models.py:GrandchildModel.end'
        ), (
            'MYSQL conflict in local_app:release_util/tests/test_check_reserved_keywords/test_app/'
            'local_app/models.py:GrandchildModel.nick_name'
        ), (
            'MYSQL conflict in local_app:release_util/tests/test_check_reserved_keywords/test_app/'
            'local_app/models.py:GrandchildModel.start'
        ), (
            'STITCH conflict in local_app:release_util/tests/test_check_reserved_keywords/test_app/'
            'local_app/models.py:GrandchildModel.end'
        ), (
            'STITCH conflict in local_app:release_util/tests/test_check_reserved_keywords/test_app/'
            'local_app/models.py:GrandchildModel.start'
        ),
    ]
    assert sorted(violation_strings) == expected_violations
    with pytest.raises(CommandError) as exception:
        set_status(violations, config)
    exc_msg = str(exception.value)
    assert "Found 5 reserved keyword conflicts!" in exc_msg


def test_reserved_keyword_detection_specific_system(django_setup):
    from .test_app.local_app import models as local_models
    load_apps(['release_util.tests.test_check_reserved_keywords.test_app.local_app'])
    model = local_models.GrandchildModel
    keyword_file = open('release_util/tests/test_check_reserved_keywords/test_files/reserved_keywords.yml', 'r')
    config = Config(keyword_file, None, 'reports', 'report.csv', 'STITCH')
    violations = check_model_for_violations(model, config)
    violation_strings = sorted([str(v) for v in violations])
    expected_violations = [
        (
            'STITCH conflict in local_app:release_util/tests/test_check_reserved_keywords/test_app/'
            'local_app/models.py:GrandchildModel.end'
        ), (
            'STITCH conflict in local_app:release_util/tests/test_check_reserved_keywords/test_app/'
            'local_app/models.py:GrandchildModel.start'
        ),
    ]
    assert sorted(violation_strings) == expected_violations


def test_overrides(django_setup):
    from .test_app.local_app import models as local_models
    model = local_models.GrandchildModel
    keyword_file = open('release_util/tests/test_check_reserved_keywords/test_files/reserved_keywords.yml', 'r')
    override_file = open('release_util/tests/test_check_reserved_keywords/test_files/overrides.yml', 'r')
    config = Config(keyword_file, override_file, 'reports', 'report.csv', None)
    violations = check_model_for_violations(model, config)
    assert len(violations) == 5
    overridden_violations = [str(v) for v in violations if v.override]
    assert overridden_violations == [
        (
            'STITCH conflict in local_app:release_util/tests/test_check_reserved_keywords/'
            'test_app/local_app/models.py:GrandchildModel.end'
        )
    ]
    with pytest.raises(CommandError) as exception:
        set_status(violations, config)
    exc_msg = str(exception.value)
    assert "Found 4 reserved keyword conflicts!" in exc_msg
